--========================================================
--           Text2Notes for Renoid v.0.14 
--           Coding:Uprime22 
-- (http://reg.s63.xrea.com/BBS/viewtopic.php?p=1433#p1433)
--=========================================================
--[[
===============
main.lua
===============
--]]


-- Thank you,KMaki,joule!
-- http://forum.renoise.com/index.php?/topic/32240-snippet-edit-step/
require ('move_cursor')

-- Load functions for indexing sample mappings
require('indexing_mappings')

-- Load functions for searching notes,etc.
require('search_notes')


-- Record a note,or do something
-- note:note value or note string or 'up','down','---'.
-- velo:volume value or volume string or nil.
-- inst:inturument-index(not value) or instrument string or nil.
-- on_off:option on/off this function. false or not-false
function note_it (note,velo,inst,on_off)
  local s = renoise.song()  
  if note =='' or note == nil
    or s.selected_note_column == nil
    or s.transport.edit_mode ~= true
    or on_off == false
  then return end
  if note =='down' then move_cursor()
  elseif note =='up' then move_cursor(- s.transport.edit_step)
  elseif note =='---' or note ==121 then
    s.selected_note_column.note_value =121
    s.selected_note_column.instrument_value =255
    s.selected_note_column.volume_value =255
  else
    if type(note) =='string' then
      s.selected_note_column.note_string = note 
    elseif type(note) =='number' then  
      s.selected_note_column.note_value = note
    end
    if type(velo) =='string' then       
      s.selected_note_column.volume_string = velo
    elseif type(velo) =='number' then
      s.selected_note_column.volume_value = velo      
    end  
    if type(inst) =='string' then       
      s.selected_note_column.instrument_string = inst
    elseif type(inst) =='number' then
      s.selected_note_column.instrument_value = inst -1    
    end
  end
end



-- Get a word from src_id,using name_index_tbl and max_letters_number,
--   and send the word to tgt_id.
-- Return the captured letters and the tbl_for_noteit
--   that is value of name_index_tbl.
-- max_letters_number:Max number of letters for searching words
-- name_lndex_tbl:Key-list. Ex. name_lndex_tbl[string_hoge] -> {note,velo,inst}
-- captured:captured words table
function get_send_txt(src_id,tgt_id,vb,name_lndex_tbl,max_letters_number,captured)
  local src = vb.views[src_id].text
  local choped = chopup(src)
  local maxs = #choped -- for word-length
  if maxs <1 then
    return
  end
  if maxs >= max_letters_number then maxs = max_letters_number
  else end
  local cap,idx = "",1
  local tbl_for_noteit ={}
  for i =maxs,1,-1 do
    local name = sub_utf(src,1,i)
    local tbl = name_lndex_tbl[name]
    if tbl ~=nil then
      cap,idx,tbl_for_noteit = name,i,tbl
      break
    else end
  end
  if cap ~="" and type(captured) == 'table' then -- capturing words
    table.insert(captured,cap)
  end   
  if cap ~="" and idx < #choped then -- when find a word,not last
    vb.views[src_id].text = sub_utf(src,idx+1)
    vb.views[tgt_id].text = table.concat(captured)
  elseif cap ~="" then -- last word
    vb.views[src_id].text = ""
    vb.views[tgt_id].text = table.concat(captured)
  elseif cap =="" and #choped >=1 then -- unknown letter
    table.remove(choped,1)
    vb.views[src_id].text = table.concat(choped)
  end
  
  return cap,tbl_for_noteit
end

-- Reverse the letters(captured) from tgt_id to src_id.  
-- captured: captured words table 
function reverse_txt(src_id,tgt_id,vb,captured)
  if (type(captured) =='table' and #captured ~= 0)
  then
    local rmvd = table.remove(captured)
    vb.views[src_id].text = tostring(rmvd)..vb.views[src_id].text
    vb.views[tgt_id].text = table.concat(captured)
  end
end


-- function for loading instruments data 
function load_insts (inst_idx,max_num_insts,max_num_letters,ini_tbl)
  local tbl = ini_tbl
  for i= inst_idx,(inst_idx +max_num_insts -1) do
    if renoise.song().instruments[i] ==nil
    then break
    else 
      tbl = 
      name2note_velo_idx (i,max_num_letters,tbl)
    end
  end
  return tbl
end


-- to play or stop the pattern-song at the line
function toggle_start (ini,line)
  if ini == false then
    ini = true    
    renoise.song().transport:start_at(line)
  elseif ini == true then
    ini = false    
    renoise.song().transport:stop()
  end
  return ini,renoise.song().selected_line_index
end

-------------------
-- GUI function
-------------------
function txt2notes ()
  local vb = renoise.ViewBuilder()

  -- trigger ini for playing a pattern
  local start_ini = false
  
  -- maximun number of letters for searching words
  local max_number_letters = 6
  
  -- maximum number of indexing instruments
  local number_of_indexing_insts = 1
  
  -- table for sample-name key indexing
  local indexs ={}
  
  -- for irregular words-notes.
  local load_irr = function ()
    indexs['.'],indexs['-'],indexs['。'],indexs['　']={},{},{},{}
    -- for Noteoff
    indexs['.'].note='OFF';indexs['。'].note='OFF'
    -- for Clearing a note.
    indexs['-'].note='---';indexs['　'].note='---' -- '　':全角スペース
  end

  -- table for captured words
  local captured_words = {}
  
  local inst_idx 
  = renoise.song().selected_instrument_index
  local inst_name
  = renoise.song().selected_instrument.name

  -- load instruments data
  indexs = load_insts (inst_idx,number_of_indexing_insts
    ,max_number_letters,indexs)
  load_irr()
  
  -- Field for source text
  local txt_src =
  vb:multiline_textfield {
    width = 400,
    height = 400,
    
    text = "",
    font = "big",
    style = "border",
    id = "txt_src"
  }

  -- Field for already recorded text
  local txt_noted =
  vb:multiline_textfield {
    width = 120,
    height = 400,
    text = "",
    font = "normal",
    style = "body",    
    id = "txt_noted"
  }

  -- Help and Settiings views

  -- Number of loading instruments,
  -- from selected instrument.
  local inst_option_txt =
  vb:text {
    text = [[Number of loading instruments : ]]
  }
  local option_number_of_insts =
  vb:valuebox {
    id = "nmbr_insts",
    value = number_of_indexing_insts,
    notifier = function (num)
      number_of_indexing_insts = num
    end,
    min = 1
  }
  -- reload keywords list
  local reload_key_btn =
  vb:button {
    id = "reload_key",
    width = 15,
    height = 15,
    text = "Reload key-list",
    notifier = function ()
      indexs = load_insts (
        renoise.song().selected_instrument_index
        ,number_of_indexing_insts
        ,max_number_letters,{})
      load_irr()
    end,
  }
  
  -- help button
  local help_txt =""
  local help =
  vb:button {
    id = "help",
    width = 10,
    height = 10,
    text = "?",
    notifier = function ()
      renoise.app():show_prompt('Key-bindings', help_txt, {'ok'})
    end
  }

  local setting_visible = false
  local setting =
  vb:column {
    visible = setting_visible,
    id = 'setting',
    vb:row {inst_option_txt,option_number_of_insts,reload_key_btn }
  }
  -- toggle setting view
  local btn_setting =
  vb:button {
    id = 'btn_setting',
    width = 10,
    height = 10,
    text = "Set",
    notifier = function ()
      if setting_visible == false then
        vb.views.setting.visible = true
        vb.views.btn_setting.text = "Close"
        setting_visible = true       
      elseif setting_visible == true then
        vb.views.setting.visible = false
        vb.views.btn_setting.text = "Set"
        setting_visible = false
      end
    end
  }    
    
    
  
  -- Functions for key-bindings
  
  -- Get a word from source text and record a note
  --   and do something.
  local get_and_noteit = function()
    local s = renoise.song()      
    if s.transport.edit_mode ~= true
      or s.selected_note_column == nil
    then
      return nil
    end
    local word,tbl=
    get_send_txt('txt_src','txt_noted',vb
    ,indexs
    ,max_number_letters,captured_words)
    local velo,note,inst
    if tbl ~= nil then
      velo =tbl.velo
      note =tbl.note 
      inst =tbl.inst
    end
    note_it (note,velo,inst)
  end

  -- reverse text,without record.
  local reverse_txt_no_note = function()
  --    undo()
    reverse_txt('txt_src','txt_noted',vb,captured_words)
  end  

  -- proceed text,without record
  local proceed_txt_no_note = function()
    local word,tbl=
    get_send_txt('txt_src','txt_noted',vb,indexs
    ,max_number_letters,captured_words)
  end

  -- move to a already recorded note and replace it
  local search_and_replace_next_note = function ()
    if (vb.views['txt_src'].text) == '' then
      return
    end
    local s = renoise.song()
    local sq = next_note_exist () 
    if sq == true and s.transport.edit_mode == true
      and s.selected_note_column ~= nil
    then
      local wrd,tbl = get_send_txt(
        'txt_src','txt_noted',vb
        ,indexs,max_number_letters,captured_words)    
      local note,velo,inst
      if wrd ~= "" then
        note =tbl.note 
        velo =tbl.velo
        inst =tbl.inst
        move_to_next_note ()
        note_it (note,velo,inst)
      end
    end
  end

  
  -- You can modify this function for Key-handling.
  help_txt =
  [[
  Esc ,Ctrl+E: Record notes mode,Edit words mode
  
  Ctrl+Enter: Get a word and record it
  
  Shift+Ctrl+Enter: Search next note and replace it
  (refering to wrapped_pattern_edit option)
  
  Ctrl+Right ,Ctrl+Left: Transpose words without recording
  
  Ctrl+Up ,Ctrl+Down: Up and down the editing line
  
  Shift+Ctrl+Up,Shift+Ctrl+Down: Search a next/previous note
  (refering to wrapped_pattern_edit option)
  
  Ctrl+1~9 ,Ctrl+Shift+1~9: Up and down the editing line with steps
  
  Ctrl+Space ,Ctrl+Shift+Space: Toggle start/stop at line 1 or current line
  ]]
  
  local key_handle = function(dialog,key)
    if
      key.modifiers == 'control' and
      key.name == 'return'
    then  get_and_noteit()
    elseif
      key.modifiers == 'shift + control' and
      key.name == 'return'
    then  search_and_replace_next_note()
    elseif
      key.modifiers == 'control' and
      key.name == 'down'
    then  note_it('down')
    elseif
      key.modifiers == 'shift + control' and
      key.name == 'down'
    then  move_to_next_note()      
    elseif
      key.modifiers == 'control' and
      key.name == 'up'
    then  note_it('up')
    elseif
      key.modifiers == 'shift + control' and
      key.name == 'up'
    then  move_to_previous_note()      
    elseif
      key.modifiers == 'control' and
      key.name == 'right'
    then  reverse_txt_no_note()
    elseif
      key.modifiers == 'control' and
      key.name == 'left'
    then  proceed_txt_no_note()      
            
    elseif
      key.modifiers == 'control' and
      type(tonumber(key.name))=='number'
    then move_cursor(tonumber(key.name))      
    elseif
      key.modifiers == 'shift + control' and
      type(tonumber(key.name))=='number'
    then move_cursor(-tonumber(key.name))
    elseif
      key.modifiers == 'control' and
      key.name == 'space'
    then start_ini = toggle_start (start_ini,1)
    elseif
      key.modifiers == 'shift + control' and
      key.name == 'space'
    then start_ini = toggle_start (start_ini,renoise.song().selected_line_index)      
    elseif
      key.modifiers == 'control' and
      key.name == 'e'
    then vb.views['txt_src'].edit_mode = true       
    end
  --[   
    print('name:'..key.name)
    print('modifiers:'..key.modifiers)
    print('character:'..tostring(key.character))
    print('note:'..tostring(key.note))
    print('repeated:'..tostring(key.repeated))
  --]]    
  end

  local s0 = vb:row{txt_noted,txt_src,vb:column{help,btn_setting,setting}}

  -- GUI appear
  renoise.app():show_custom_dialog("Text2Notes".."  "..inst_name ,s0,key_handle)
  
end

--[
-- menu entry            
renoise.tool():add_menu_entry {
   name = "Pattern Editor:Text2Notes forRenoid...",
   invoke = function() txt2notes() end
}

--]]
